#!/usr/bin/perl

=head1 NAME

fai-ts - FAI Template System

=head1 SYNOPSIS

 fai-ts

=head1 ABSTRACT

Collect configuration data and fill templates to create config files. 

=head1 DESCRIPTION

fai-ts searches for '.yml' files in the 'class' directory of the fai config space, respecting fai classes. It searches for template files in the 'templates' directory of the fai config space to fill this data in, again respecting fai classes. The result will be copied to the install client.

Not yet implemented:

User, group and permissions for the files are handeled similar to fcopy (1). The file 'file-modes' in the template directory can contain one line for each class.  The space separated parameters are:

owner group mode class

where  owner  and group can be specified by numeric id or by name.  The file permissions are defined by mode, which must be octal.  If class is missing,  the  data is used for all other classes, for which no data is defined.  If neither -m or file-modes  data  is  available,  the  user, group and mode of the source file will be used for the target file.

Missing  subdirectories for the target file are created with mode 0755.  If the destination file already exists and is different from the source file,  the suffix .pre_fcopy is appended to the old version of the target file.  If source and target files are equal, no copy is  performed.  A  copied file gets the same access and modification time as the source file, not the time of the copy.  If also the file postinst  exists,  it will be executed after the file was copied.

=head1 ENVIROMENT

The following enviroment variables are used:

 $classes : FAI classes
 $FAI     : path to fai config space
 $target  : path to install client root

=cut

BEGIN 
{
  unless ( $ENV{FAI} and $ENV{classes} and $ENV{target} )
  { 
    print STDERR 'This should be called from within fai', $/;
    exit 1;
  }
}

use strict;
use warnings;

use lib "$ENV{FAI}/lib";

use Getopt::Long;
use Pod::Usage;
use YAML qw(LoadFile);
use Hash::Merge qw(merge);
use FaiTS qw(writeTargetFiles findTemplates);

my %opts;
GetOptions( \%opts, 'help|?', 'man' ) or pod2usage(1);
pod2usage(1) if $opts{help};
pod2usage(-verbose => 2) if $opts{man};

my $prefixTemplateDir = "$ENV{FAI}/templates";
my $prefixTargetDir   = "$ENV{target}";
my $prefixYmlDir      = "$ENV{FAI}/class";
my %ignoreDirs        = map { $_, 1 } qw/CVS .svn .arch-ids {arch}/;
my @classes           = split /\s+/, $ENV{classes};
my @ymlFiles          = grep { -f "$prefixYmlDir/$_.yml" } @classes;
@ymlFiles             = map { $_ = "$prefixYmlDir/$_.yml" } @ymlFiles;

die "$prefixTemplateDir missing" 
  unless -d $prefixTemplateDir;

#Read config from ENV and @ymlFiles
my $config = { %ENV };
$config = merge( $config, LoadFile($_) || die $! ) 
  foreach @ymlFiles;

#Search for templates to fill
my $templates = findTemplates( \$ignoreDirs, \@classes, $prefixTemplateDir);

# process templates and write them to $ENV{target}
writeTargetFiles($templates, \@classes, $config, $prefixTemplateDir, $prefixTargetDir);

# ------------- end main ------------------------------

=head1 SECURITY

Since this program may be used during softupdates in a running system, it refuses to write to directories writable by others than the owner. But beware: more security implications may exist ... the author cannot be made responsible for any of these.

=head1 SEE ALSO

 http://www.yaml.org
 http://www.template-toolkit.org
 http://www.informatik.uni-koeln.de/fai/

=head1 AUTHOR
 
 Copyright 2006 by Ingo Wichmann <iw@linuxhotel.de>

 This software ist free software; you can redistribute it and/or modify
 it unter the same terms as Perl itself.

=cut

